/*
 * Copyright (c) 2006-2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <GPSNovatel/GPSNovatel.h>
#include <GPSNovatel/sampleAPI.h>
#include "gpx.h"
#include "serial.h"
#include "logData.h"

// #define DEBUG

static char *logFile;
static int verboseLevel;


// Descripteur du fichier de log
static FILE *fdLog;
// verbose or not
GPSNovatel_FLAG verbose;
// the format of logging
GPS_LOG_FORMAT format;

/*
  log utm
 */
static void
logUTM(const OEM4_TRAME_COMPUTED *frame, FILE *out)
{

	if (out==NULL)
		return;

	fprintf(out,
	    "\nBESTUTMb %u %u %u %u %u %c %.15f %.15f %.15f %.15f %u %.15f"
	    " %.15f %.15f %u %u %u %u %.15f %.15f %u %u %u %u\n",
	    (unsigned int)frame->timev.tv_sec,
	    (unsigned int)frame->timev.tv_usec,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.solStatus,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.posType,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.longitudZoneNumber,
	    (char)frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.latitudinalZoneLetter,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.northing,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.easting,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.height,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.undulation,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.datumID,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.northingDev,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.eastingDev,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.heightDev,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.baseStationID[0],
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.baseStationID[1],
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.baseStationID[2],
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.baseStationID[3],
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.differentialAge,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.solutionAge,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.nbSatTracked,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.nbGPSL1Used,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.nbGPSL1,
	    frame->TRAME_TYPE.strOEM4BESTUTMb_LOG.nbGPSL2);

}

/*
  verbose log containing label before utm fields
*/
void
logVerbose(OEM4BESTUTMb_LOG *utm,FILE *out)
{
	if (out==NULL)
		return;

	fprintf(out,"\n\nMESSAGE BESTUTMb\n");
	fprintf(out,"Solution status = %u\n", utm->solStatus);
	fprintf(out,"Position type = %u\n",utm->posType);
	fprintf(out,"Longitudinal zone number  = %u\n", utm->longitudZoneNumber);
	fprintf(out,"Latitudinal zone letter = %u\n", utm->latitudinalZoneLetter);
	fprintf(out,"Northing origin = %.15f\n",utm->northing);
	fprintf(out,"Easting origin = %.15f\n", utm->easting);
	fprintf(out,"Height above mean sea level = %.15f\n", utm->height);
	fprintf(out,"Northing st. deviation = %.15f\n",utm->undulation);
	fprintf(out,"Height st. deviation = %o\n",utm->datumID);
	fprintf(out,"Easting origin = %.15f\n", utm->northingDev);
	fprintf(out,"Undulation = %.15f\n", utm->eastingDev);
	fprintf(out,"Datum ID number = %e\n", utm->heightDev);
	fprintf(out,"Base station ID.0 = %u\n", utm->baseStationID[0]);
	fprintf(out,"Base station ID.1 = %u\n", utm->baseStationID[1]);
	fprintf(out,"Base station ID.2 = %u\n", utm->baseStationID[2]);
	fprintf(out,"Base station ID.3 = %u\n", utm->baseStationID[3] );
	fprintf(out,"Differential age in seconds = %.15f\n", utm->differentialAge);
	fprintf(out,"Solution age in seconds = %.15f\n",utm->solutionAge );
	fprintf(out,"Number of satellite tracked = %u\n", utm->nbSatTracked );
	fprintf(out,"Number GPS L1 ranges used in computation = %u\n", utm->nbGPSL1Used);
	fprintf(out,"Number of GPS L1 ranges above RTK mask angle = %u\n", utm->nbGPSL1);
	fprintf(out,"Number of GPS L2 ranges above RTK mask angle = %u\n",utm->nbGPSL2);
	fprintf(out,"FIN MESSAGE\n");
}

static
void logSpeed(const OEM4_TRAME_COMPUTED *frame, FILE *out)
{

	if (out==NULL)
		return;

	fprintf(out,
	    "\nBESTVELb %u %u %u %u %.15f %.15f %.15f %.15f %.15f\n",
	    (unsigned int)frame->timev.tv_sec,
	    (unsigned int)frame->timev.tv_usec,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.solStatus,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.velocityType,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.latency,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.age,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.horizontalSpeed,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.actualDirection,
	    frame->TRAME_TYPE.strOEM4BESTVELb_LOG.verticalSpeed);
}

void
initLogGPX(FILE* out)
{
	print_gpx_header (out);
	print_gpx_trk_start(out);

}

#include "h2timeLib.h"

/*
  log to GPX format
 */
void
logGPX(OEM4BESTPOSb_LOG *pos, FILE* out)
{
	gps_fix_t gpx_pos;
	struct tm time;

	H2TIME h2Time;
	/* get date */
	h2timeGet(&h2Time);
	time.tm_year=h2Time.year;
	time.tm_mon=h2Time.month;
	time.tm_mday=h2Time.day;
	time.tm_hour=h2Time.hour;
	time.tm_min=h2Time.minute;
	time.tm_sec=h2Time.sec;

	gpx_pos.latitude = pos->lat;
	gpx_pos.longitude = pos->lon;
	gpx_pos.altitude = pos->hgt;

	print_fix(&gpx_pos,&time,out);
}

void
endLogGPX(FILE* out) {
	print_gpx_trk_end(out);
	print_gpx_footer(out);
}

void
logVel(void)
{

}

/*!
  if filename=
  _verboseLevel : details of logging
*/
void
GPSNovatel_setLogData(GPSNovatel_FLAG _verbose, int _verboseLevel, char *_fileName, GPS_LOG_FORMAT _format)
{
	format=_format;
	fdLog = NULL;
	verbose=_verbose;

	logFile = _fileName;
	verboseLevel = _verboseLevel;

	if (format == GPS_LOG_NONE)
		return;

	if (_fileName != NULL) {
		verbose=NOV_TRUE;
		if ((fdLog=fopen(logFile, "w+")) == NULL) {
			fprintf(stderr, "open log %s :%s\n", logFile, strerror(errno));
		}
	}

	if ((format==GPS_LOG_GPX) && (verbose==NOV_TRUE))  {
		if (fdLog!=NULL)
			initLogGPX(fdLog);
		else
			initLogGPX(stdout);
	}
}

void
GPSNovatel_endLogData()
{
	if ((format==GPS_LOG_GPX) && (verbose==NOV_TRUE)) {
		if (fdLog!=NULL)
			endLogGPX(fdLog);
		else
			endLogGPX(stdout);
	}
}

void
GPSNovatel_logData(void)
{
	int i = 0;
	FILE* dest;

#ifdef DEBUG
	printf("[GPSNovatel] logData : NB TRAME = %d\n", NB_TRAME_BUFFERED);
#endif

	// Lecture des informations du GPS et log dans un fichier
	if (fdLog != NULL) {
		dest=fdLog;
	} else
		if (verbose == NOV_TRUE)
			dest = stdout;
		else
			return;

	while (i < NB_TRAME_BUFFERED) {

		/* log operation */
		switch (format) {
		case GPS_LOG_UTM :
			if (USR_TRAME[i].BINARY_HEADER.number==BESTUTMb)
				logUTM(&USR_TRAME[i],dest);
			break;
		case GPS_LOG_GPX :
			if (USR_TRAME[i].BINARY_HEADER.number==BESTPOSb)
				logGPX(&(USR_TRAME[i].TRAME_TYPE.strOEM4BESTPOSb_LOG),dest);
			break;
		case GPS_LOG_VERBOSE :
			if (USR_TRAME[i].BINARY_HEADER.number==BESTUTMb)
				logVerbose(&(USR_TRAME[i].TRAME_TYPE.strOEM4BESTUTMb_LOG),dest);
			break;
		default :
			break;
		}  /* switch */

		if (USR_TRAME[i].BINARY_HEADER.number == BESTVELb)
			logSpeed(&USR_TRAME[i], dest);

		i++;

	} // while

	if (fdLog != NULL)
		fflush(fdLog);
}
